/* -*- mode:c; coding:utf-8; tab-width:8; c-basic-offset:2; indent-tabs-mode:nil -*- */
/*
  Copyright (c) 2004 MacUIM Project
  http://www.digital-genes.com/~yatsu/macuim/

  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

  1. Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
  2. Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  3. Neither the name of authors nor the names of its contributors
     may be used to endorse or promote products derived from this software
     without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  SUCH DAMAGE.
*/

#ifndef MUIM_h
#define MUIM_h

#include <Carbon/Carbon.h>

#include <uim/uim.h>
#include <uim/uim-helper.h>

#include "Utils.h"
#include "Debug.h"

typedef struct _PreeditSegment
{
  UniCharPtr fBuffer;
  UInt32 fLength;
  UInt32 fAttr;

} PreeditSegment;

typedef struct _MUIMSessionRecord
{
  ComponentInstance fComponentInstance;
  PreeditSegment *fSegments;
  UInt32 fSegmentCount;

  //uim_context fUC;

  UInt32 fLastUpdateLength;
    
  UniCharPtr fFixBuffer;
  UInt32 fFixLen;

  UniCharPtr fOldPreedit;
  UInt32 fOldPreeditLen;

  SInt32 fNRCandidates;
  SInt32 fCandidateIndex;
  SInt32 fLayoutBegin;
  UInt32 fIsActive;
  SInt32 fDisplayLimit;
  SInt32 fSelectCount;

  CFBundleRef fBundleRef;
  Boolean fWindowOpened;

} MUIMSessionRecord;

typedef MUIMSessionRecord *MUIMSessionPtr;
typedef MUIMSessionPtr *MUIMSessionHandle;

typedef struct _TSMStat {
  long fRefCon;
  long fTextOffset;
  ScriptLanguageRecord fSLRec;
  Boolean fLeadingEdge;
  Point fReplyPoint;
  long fReplyFont;
  Fixed fReplyPointSize;
  short fReplyLineHeight;

} TSMStat;


ComponentResult
MUIMInitialize(ComponentInstance inComponentInstance,
               MenuRef *outTextServiceMenu);

void
MUIMTerminate(ComponentInstance inComponentInstance);

ComponentResult
MUIMSessionOpen(ComponentInstance inComponentInstance,
                MUIMSessionHandle *outSessionHandle);

void
MUIMSessionClose(MUIMSessionHandle sessionHandle);

ComponentResult
MUIMSessionActivate(MUIMSessionHandle sessionHandle);

ComponentResult
MUIMSessionDeactivate(MUIMSessionHandle sessionHandle);

ComponentResult
MUIMSessionEvent(MUIMSessionHandle sessionHandle, EventRef eventRef);

/*
ComponentResult
MUIMSessionFix(MUIMSessionHandle sessionHandle);
*/

MUIMSessionHandle
MUIMGetActiveSession(void);

Boolean
MUIMHandleInput(MUIMSessionHandle inSessionHandle, UInt32 inKeycode, unsigned char inCharCode, UInt32 inModifiers);

void
GetPreeditSegment(PreeditSegment *inSegment, UniCharPtr *outStr,
                  UInt32 *outLen);

void
GetPreeditString(MUIMSessionHandle inHandle, UniCharPtr *outStr,
                 UInt32 *outLen, UInt32 *outCursorPos);

void
SegmentToHiliteRange(MUIMSessionHandle inHandle, TextRangeArrayPtr *inRange);

void
DumpString(const char *name, const char *str, int len);

#endif // MUIM_h
